# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Pazar',
        1: 'Pazartesi',
        2: 'Salı',
        3: 'Çarşamba',
        4: 'Perşembe',
        5: 'Cuma',
        6: 'Cumartesi'
    },
    'days_abbrev': {
        0: 'Paz',
        1: 'Pzt',
        2: 'Sal',
        3: 'Çar',
        4: 'Per',
        5: 'Cum',
        6: 'Cmt'
    },

    # Months
    'months': {
        1: 'Ocak',
        2: 'Şubat',
        3: 'Mart',
        4: 'Nisan',
        5: 'Mayıs',
        6: 'Haziran',
        7: 'Temmuz',
        8: 'Ağustos',
        9: 'Eylül',
        10: 'Ekim',
        11: 'Kasım',
        12: 'Aralık',
    },
    'months_abbrev': {
        1: 'Oca',
        2: 'Şub',
        3: 'Mar',
        4: 'Nis',
        5: 'May',
        6: 'Haz',
        7: 'Tem',
        8: 'Ağu',
        9: 'Eyl',
        10: 'Eki',
        11: 'Kas',
        12: 'Ara',
    },

    # Units of time
    'year': '{count} yıl',
    'month': '{count} ay',
    'week': '{count} hafta',
    'day': '{count} gün',
    'hour': '{count} saat',
    'minute': '{count} dakika',
    'second': '{count} saniye',

    # Relative time
    'ago': '{time} önce',
    'from_now': '{time} sonra',
    'after': '{time} sonra',
    'before': '{time} önce',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, D MMMM YYYY HH:mm',
        'LLL': 'D MMMM YYYY HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD.MM.YYYY',
    },
}
