# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Якшанба',
        1: 'Душанба',
        2: 'Сешанба',
        3: 'Чоршанба',
        4: 'Пайшанба',
        5: 'Жума',
        6: 'Шанба'
    },
    'days_abbrev': {
        0: 'Якш',
        1: 'Душ',
        2: 'Сеш',
        3: 'Чор',
        4: 'Пай',
        5: 'Жум',
        6: 'Шан'
    },

    # Months
    'months': {
        1: 'январ',
        2: 'феврал',
        3: 'март',
        4: 'апрел',
        5: 'май',
        6: 'июн',
        7: 'июл',
        8: 'август',
        9: 'сентябр',
        10: 'октябр',
        11: 'ноябр',
        12: 'декабр'
    },
    'months_abbrev': {
        1: 'янв',
        2: 'фев',
        3: 'мар',
        4: 'апр',
        5: 'май',
        6: 'июн',
        7: 'июл',
        8: 'авг',
        9: 'сен',
        10: 'окт',
        11: 'ноя',
        12: 'дек'
    },

    # Units of time
    'year': ['{count} yil', '{count} yil', '{count} yil'],
    'month': ['{count} oy', '{count} oy', '{count} oylar'],
    'week': ['{count} hafta', '{count} hafta', '{count} hafta'],
    'day': ['{count} kun', '{count} kun', '{count} kun'],
    'hour': ['{count} soat', '{count} soat', '{count} soat'],
    'minute': ['{count} minut', '{count} minut', '{count} minut'],
    'second': ['{count} sekund', '{count} sekund', '{count} sekund'],

    # Relative time
    'ago': '{time} avval',
    'from_now': 'keyin {time}',
    'after': '{time} keyin',
    'before': '{time} gacha',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'D MMMM YYYY, dddd HH:mm',
        'LLL': 'D MMMM YYYY HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD/MM/YYYY',
    },
}
