# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'niedziela',
        1: 'poniedziałek',
        2: 'wtorek',
        3: 'środa',
        4: 'czwartek',
        5: 'piątek',
        6: 'sobota'
    },
    'days_abbrev': {
        0: 'Nd',
        1: 'Pn',
        2: 'Wt',
        3: 'Śr',
        4: 'Czw',
        5: 'Pt',
        6: 'So'
    },

    # Months
    'months': {
        1: 'styczeń',
        2: 'luty',
        3: 'marzec',
        4: 'kwiecień',
        5: 'maj',
        6: 'czerwiec',
        7: 'lipiec',
        8: 'sierpień',
        9: 'wrzesień',
        10: 'październik',
        11: 'listopad',
        12: 'grudzień',
    },
    'months_abbrev': {
        1: 'sty',
        2: 'lut',
        3: 'mar',
        4: 'kwi',
        5: 'maj',
        6: 'cze',
        7: 'lip',
        8: 'sie',
        9: 'wrz',
        10: 'paź',
        11: 'lis',
        12: 'gru',
    },

    # Units of time
    'year': ['{count} rok', '{count} lata', '{count} lat'],
    'month': ['{count} miesiąc', '{count} miesiące', '{count} miesięcy'],
    'week': ['{count} tydzień', '{count} tygodnie', '{count} tygodni'],
    'day': ['{count} dzień', '{count} dni', '{count} dni'],
    'hour': ['{count} godzina', '{count} godziny', '{count} godzin'],
    'minute': ['{count} minuta', '{count} minuty', '{count} minut'],
    'second': ['{count} sekunda', '{count} sekundy', '{count} sekund'],

    # Relative time
    'ago': '{time} temu',
    'from_now': '{time} od teraz',
    'after': '{time} po',
    'before': '{time} przed',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, D MMMM YYYY HH:mm',
        'LLL': 'D MMMM YYYY HH:mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD.MM.YYYY',
    },
}
