# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Chủ Nhật',
        1: 'Thứ Hai',
        2: 'Thứ Ba',
        3: 'Thứ Tư',
        4: 'Thứ Năm',
        5: 'Thứ Sáu',
        6: 'Thứ Bảy'
    },
    'days_abbrev': {
        0: 'CN',
        1: 'Thứ 2',
        2: 'Thứ 3',
        3: 'Thứ 4',
        4: 'Thứ 5',
        5: 'Thứ 6',
        6: 'Thứ 7'
    },

    # Months
    'months': {
        1: 'Tháng Một',
        2: 'Tháng Hai',
        3: 'Tháng Ba',
        4: 'Tháng Tư',
        5: 'Tháng Năm',
        6: 'Tháng Sáu',
        7: 'Tháng Bảy',
        8: 'Tháng Tám',
        9: 'Tháng Chín',
        10: 'Tháng Mười',
        11: 'Tháng Mười Một',
        12: 'Tháng Mười Hai',
    },
    'months_abbrev': {
        1: 'Tháng 1',
        2: 'Tháng 2',
        3: 'Tháng 3',
        4: 'Tháng 4',
        5: 'Tháng 5',
        6: 'Tháng 6',
        7: 'Tháng 7',
        8: 'Tháng 8',
        9: 'Tháng 9',
        10: 'Tháng 10',
        11: 'Tháng 11',
        12: 'Tháng 12',
    },

    # Units of time
    'year': '{count} năm',
    'month': '{count} tháng',
    'week': '{count} tuần',
    'day': '{count} ngày',
    'hour': '{count} giờ',
    'minute': '{count} phút',
    'second': '{count} giây',

    # Relative time
    'ago': '{time} trước',
    'from_now': '{time} từ bây giờ',
    'after': '{time} sau',
    'before': '{time} trước',

    # Date formats
    'date_formats': {
        'LTS': 'HH:mm:ss',
        'LT': 'HH:mm',
        'LLLL': 'dddd, D MMMM [năm] YYYY HH:mm',
        'LLL': 'D MMMM [năm] YYYY HH:mm',
        'LL': 'D MMMM [năm] YYYY',
        'L': 'DD/MM/YYYY',
    },
}
