# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'pühapäev',
        1: 'esmaspäev',
        2: 'teisipäev',
        3: 'kolmapäev',
        4: 'neljapäev',
        5: 'reede',
        6: 'laupäev'
    },
    'days_abbrev': {
        0: 'P',
        1: 'E',
        2: 'T',
        3: 'K',
        4: 'N',
        5: 'R',
        6: 'L'
    },

    # Months
    'months': {
        1: 'jaanuar',
        2: 'veebruar',
        3: 'märts',
        4: 'aprill',
        5: 'mai',
        6: 'juuni',
        7: 'juuli',
        8: 'august',
        9: 'september',
        10: 'oktoober',
        11: 'november',
        12: 'detsember',
    },
    'months_abbrev': {
        1: 'jaan',
        2: 'veebr',
        3: 'märts',
        4: 'apr',
        5: 'mai',
        6: 'juuni',
        7: 'juuli',
        8: 'aug',
        9: 'sept',
        10: 'okt',
        11: 'nov',
        12: 'dets',
    },

    # Units of time
    'year': ['{count} aasta', '{count} aastat'],
    'month': ['{count} kuu', '{count} kuud'],
    'week': ['{count} nädal', '{count} nädalat'],
    'day': ['{count} päev', '{count} päeva'],
    'hour': ['{count} tund', '{count} tundi'],
    'minute': ['{count} minut', '{count} minutit'],
    'second': ['{count} sekund', '{count} sekundit'],

    # Relative time
    'ago': '{time} tagasi',
    'from_now': '{time} pärast',
    'after': '{time} pärast',
    'before': '{time} enne',
    'year_from_now': '{count} aasta',
    'month_from_now': '{count} kuu',
    'week_from_now': '{count} nädala',
    'day_from_now': '{count} päeva',
    'hour_from_now': '{count} tunni',
    'minute_from_now': '{count} minuti',
    'second_from_now': '{count} sekundi',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LLLL': 'dddd, D. MMMM YYYY H:mm',
        'LLL': 'dddd, D. MMMM YYYY H:mm',
        'LL': 'dddd, D. MMMM YYYY H:mm',
        'L': 'dddd, D. MMMM YYYY H:mm',
    },
}
