# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Nedjelja',
        1: 'Ponedjeljak',
        2: 'Utorak',
        3: 'Srijeda',
        4: 'Četvrtak',
        5: 'Petak',
        6: 'Subota'
    },
    'days_abbrev': {
        0: 'Ne',
        1: 'Po',
        2: 'Ut',
        3: 'Sr',
        4: 'Če',
        5: 'Pe',
        6: 'Su'
    },

    # Months
    'months': {
        1: 'Siječanj',
        2: 'Veljača',
        3: 'Ožujak',
        4: 'Travanj',
        5: 'Svibanj',
        6: 'Lipanj',
        7: 'Srpanj',
        8: 'Kolovoz',
        9: 'Rujan',
        10: 'Listopad',
        11: 'Studeni',
        12: 'Prosinac',
    },
    'months_abbrev': {
        1: 'Sij',
        2: 'Velj',
        3: 'Ožu',
        4: 'Tra',
        5: 'Svi',
        6: 'Lip',
        7: 'Srp',
        8: 'Kol',
        9: 'Ruj',
        10: 'Lis',
        11: 'Stu',
        12: 'Pro',
    },

    # Units of time
    'year': ['{count} godinu', '{count} godine', '{count} godina'],
    'month': ['{count} mjesec', '{count} mjeseca', '{count} mjeseci'],
    'week': ['{count} tjedan', '{count} tjedna', '{count} tjedana'],
    'day': ['{count} dan', '{count} dana', '{count} dana'],
    'hour': ['{count} sat', '{count} sata', '{count} sati'],
    'minute': ['{count} minutu', '{count} minute ', '{count} minuta'],
    'second': ['{count} sekundu', '{count} sekunde', '{count} sekundi'],

    # Relative time
    'ago': 'prije {time}',
    'from_now': 'za {time}',
    'after': 'za {time}',
    'before': 'prije {time}',

    # Ordinals
    'ordinal': '.',

    # Date formats
    'date_formats': {
        'LTS': 'H:mm:ss',
        'LT': 'H:mm',
        'LLLL': 'dddd, D. MMMM YYYY H:mm',
        'LLL': 'D. MMMM YYYY H:mm',
        'LL': 'D. MMMM YYYY',
        'L': 'DD. MM. YYYY',
    },
}
