# -*- coding: utf-8 -*-

translations = {
    # Days
    'days': {
        0: 'Minggu',
        1: 'Senin',
        2: 'Selasa',
        3: 'Rabu',
        4: 'Kamis',
        5: 'Jumat',
        6: 'Sabtu'
    },
    'days_abbrev': {
        0: 'Min',
        1: 'Sen',
        2: 'Sel',
        3: 'Rab',
        4: 'Kam',
        5: 'Jum',
        6: 'Sab'
    },

    # Months
    'months': {
        1: 'Januari',
        2: 'Februari',
        3: 'Maret',
        4: 'April',
        5: 'Mei',
        6: 'Juni',
        7: 'Juli',
        8: 'Agustus',
        9: 'September',
        10: 'Oktober',
        11: 'November',
        12: 'Desember'
    },
    'months_abbrev': {
        1: 'Jan',
        2: 'Feb',
        3: 'Mar',
        4: 'Apr',
        5: 'Mei',
        6: 'Jun',
        7: 'Jul',
        8: 'Ags',
        9: 'Sep',
        10: 'Okt',
        11: 'Nov',
        12: 'Des'
    },

    # Units of time
    'year': '{count} tahun',
    'month': '{count} bulan',
    'week': '{count} minggu',
    'day': '{count} hari',
    'hour': '{count} jam',
    'minute': '{count} menit',
    'second': '{count} detik',

    # Relative time
    'ago': '{time} yang lalu',
    'from_now': '{time} dari sekarang',
    'after': '{time} setelah',
    'before': '{time} sebelum',

    # Date formats
    'date_formats': {
        'LTS': 'HH.mm.ss',
        'LT': 'HH.mm',
        'LLLL': 'dddd, D MMMM YYYY [pukul] HH.mm',
        'LLL': 'D MMMM YYYY [pukul] HH.mm',
        'LL': 'D MMMM YYYY',
        'L': 'DD/MM/YYYY',
    },
}
